package it.eng.spago.workflow.api;

import java.util.Map;





/**
 * @author zoppello
 *
 * An Assignment is an activity of a process associated to a connection, potentially
 * we can say that an assignment is an activity that CAN BE executed by the user of the
 * current IWorkflowConnection.
 * 
 * Pay attention that this dosn't means that the user MUST DO the activity but only that it can.
 * This is because activity assignment in workflow engines are often associated to the user roles.
 * 
 * If a user accept an activity, with accept then the assignment is associated with user and other users with
 * the same roles doesn't see this assignment anymore.
 * 
 */
public interface IWorkflowAssignment {
	
	/**
	 * This method return return the key of the process that own the assignment
	 * @return return the key of the process at runtime
	 * @throws WorkflowEngineException if some error occurs
	 */
	public String getProcessKey() throws WorkflowEngineException;
	
	/**
	 * This method return the name of the process that own the assignment
	 * @return the name of the process that own the assignment
	 * @throws WorkflowEngineException if some error occurs
	 */
	public String getProcessName() throws WorkflowEngineException;
	
	/**
	 * This method return the description of the process at runtime
	 * @return the description of the process that own the assignment
	 * @throws WorkflowEngineException if some error occurs
	 */
	public String getProcessDescription() throws WorkflowEngineException;
	
	/**
	 * This method return the state of the process at runtime
	 * @return the state of the process that own the assignment
	 * @throws WorkflowEngineException if some error occurs
	 */
	public String getProcessState() throws WorkflowEngineException;
	
	/**
	 * This method return the key of the activity for the assignment
	 * @return the key of the activity for the assignment
	 * @throws WorkflowEngineException if some error occurs
	 */
	public String getActivityKey() throws WorkflowEngineException;;
	
	/**
	 * This method return the name of the activity for the assignment
	 * @return the name of the activity for the assignment
	 * @throws WorkflowEngineException if some error occurs
	 */
	public String getActivityName() throws WorkflowEngineException;
	
	/**
	 * This method return the description of the activity for the assignment
	 * @return the description of the activity for the assignment
	 * @throws WorkflowEngineException if some error occurs
	 */
	public String getActivityDescription() throws WorkflowEngineException;
	
	/**
	 * This method return the priority of the activity for the assignment
	 * @return the priority of the activity for the assignment
	 * @throws WorkflowEngineException if some error occurs
	 */
	public short getActivityPriority() throws WorkflowEngineException;
	
	/**
	 * This method return the state of the activity for the assignment
	 * @return the state of the activity for the assignment
	 * @throws WorkflowEngineException if some error occurs
	 */
	public String getActivityState() throws WorkflowEngineException;
	
	/**
	 * @return true if the assignment have been accepted by some user
	 * @throws WorkflowEngineException if some error occurs
	 */
	public boolean isAccepted() throws WorkflowEngineException;
	
	/**
	 * This method complete the assignment
	 * @throws WorkflowEngineException if some error occurs
	 */
	public void complete() throws WorkflowEngineException;
	
	/**
	 * This method abort the assignment
	 * @throws WorkflowEngineException if some error occurs
	 */
	public void abort() throws WorkflowEngineException;
	
	/**
	 * This method return the Context of the process that own the assignment
	 * @return the Context of the process that own the assignment
	 * @throws WorkflowEngineException if some error occurs
	 */
	public Map getContext() throws WorkflowEngineException;;
	
	/**
	 * @param context the new Context to set in process
	 * @throws WorkflowEngineException if some error occurs
	 */
	public void setContext(Map context) throws WorkflowEngineException;
	
	/**
	 * This ensure that user in connection take the assignment
	 * @throws WorkflowEngineException if some error occurs
	 */
	public void accept()  throws WorkflowEngineException;
	
	/**
	 * In SpagoWorkflow API an assignment can be associate to a form that must be completed
	 * to complete assignment
	 * @return the identifier of the form related to an assignment
	 */
	public String getMappedForm() ;
	
	/**
	 * If an assignment have been taken and cannot be complete by a user this method can be called 
	 * and the assignemnt become available to other users that can be complete it
	 * @throws WorkflowEngineException if some error occurs
	 */
	public void reject() throws WorkflowEngineException;
		
}
