package it.eng.spago.workflow.api;

import java.util.List;
import java.util.Map;

/**
 * @author zoppello
 *
 * This is a very important interface because it's the core interface that a spago application 
 * must use to work with workflow engine.
 * 
 * With this interface it's possible to:
 * 
 * <ul>
 * <li> Open the connection to the engine after we've obtained from IWorkflowEngine</li>
 * <li> Istantiate a Process for a given packageId and a given workflowId, a package can contain more than one workflow defintion</li>
 * <li> Istantiate a Process for a given packageId and a given workflowId with a specified initial context</li>
 * <li> Getting the assignments for a user ( see IWorkflowAssignment ) </li>
 * <li> Getting the assignment for a given activity Id ( An assignment is an activity assigned to a user ) </li>
 * <li> Delete a process from workflow engine </li>
 * <li> Get all workflow process definition for a given package </li>
 * <li> Get all workflow process definition </li>
 * </ul>
 */
public interface IWorkflowConnection {
	
	
	/**
	 * This method establish the connection with the engine using the user and passord credential
	 * 
	 * @param userId The user etsablishing connection with the engine
	 * @param userPassword The password of the user etsablishing connection with the engine
	 * @throws WorkflowEngineException if some errors occurs
	 */
	public void open(String userId, String userPassword) throws WorkflowEngineException;
	
	
	
	/**
	 * Crate an instance of the Process with the given packageId and workflowId
	 * @param workflowPackageID The package Identifier
	 * @param workflowID The workflow identifier
	 * @return The instance of the IWorkflowProcess representing the process instance
	 * @throws WorkflowEngineException if some errors occurs
	 */
	public IWorkflowProcess createProcess(String workflowPackageID, String workflowID) throws WorkflowEngineException;
	
	/**
	 * Crate an instance of the Process with the given packageId and workflowId and the given context
	 * @param workflowPackageID The package Identifier
	 * @param workflowID The workflow identifier
	 * @param context the initial context of the process
	 * @return  The instance of the IWorkflowProcess representing the process instance, intialized with the given context
	 * @throws WorkflowEngineException if some errors occurs
	 */
	public IWorkflowProcess createProcessWithInitialContext(String workflowPackageID, String workflowID, Map context) throws WorkflowEngineException;
	
	/**
	 * The List assignemt activity associated with the user that has establishe the connection
	 * 
	 * @return The List assignemt activity associated with the user that has establishe the connection
	 * @throws WorkflowEngineException if some errors occurs
	 */
	public List getAssignments() throws WorkflowEngineException;

	/**
	 * Interrupt the connection with the engine
	 * 
	 * @throws WorkflowEngineException
	 */
	public void close() throws WorkflowEngineException;
	
	/**
	 * List assignemt activity associated with the user that has establishe the connection
	 * 
	 * @return The List assignemt activity associated with the user that has establishe the connection
	 * @throws WorkflowEngineException if some errors occurs
	 * @param activityId The activity Id
	 * @return The Assignment for the givena ctivity ID
	 * @throws WorkflowEngineException
	 */
	public IWorkflowAssignment getWorkflowAssignment(String activityId) throws WorkflowEngineException;

	/**
	 * Given the package identifier return the list of the runtime process for the package
	 * @param pkg  The package identifier
	 * @return the list of the runtime process for the package
	 * @throws WorkflowEngineException
	 */
	public List getProcessForPackage(IWorkflowPackage pkg) throws WorkflowEngineException;
	
	/**
	 * Delete a process instance from the engine
	 * @param process the process to delete
	 * @throws WorkflowEngineException
	 */
	public void deleteProcess(IWorkflowProcess process) throws WorkflowEngineException;
	
	/**
	 * The list of the Workflow defintion object in the system
	 * @return the list of th IWorkflowDefinition objects in the engine
	 * @throws WorkflowEngineException
	 */
	public  List getAllWorkflowProcessDefinition() throws WorkflowEngineException;
	
	/**
	 * @param pkg The IWorkflowPackage
	 * @return  the list of the IWorkflowDefinition objects in the engine for the given package
	 * @throws WorkflowEngineException
	 */
	public List getWorkflowProcessDefinition(IWorkflowPackage pkg) throws WorkflowEngineException;
}
